package com.jiuqi.util;

import java.util.StringTokenizer;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.io.Serializable;

public class StringList implements Serializable, Cloneable
{
    private static final int DELIMITER_DEFINED = 1;
    private static final int QUOTECHAR_DEFINED = 2;
    private static final int NAMEVALSEP_DEFINED = 4;
    protected List items;
    protected List objects;
    private char delimiter;
    private char quoteChar;
    private char nameValueSeparator;
    private int defined;
    private static final String CS_GB2312 = "GB2312";
    private static final String CS_ISO_8859_1 = "ISO-8859-1";
    
    public StringList() {
        this.items = new ArrayList();
        this.objects = null;
        this.delimiter = '\0';
        this.quoteChar = '\0';
        this.nameValueSeparator = '\0';
        this.defined = 0;
    }
    
    protected boolean getDefined(final int item) {
        return (this.defined & item) != 0x0;
    }
    
    protected void setDefined(final int item) {
        this.defined |= item;
    }
    
    public boolean isEmpty() {
        return this.items.size() == 0;
    }
    
    public int size() {
        return this.items.size();
    }
    
    public void clear() {
        this.items.clear();
        if (this.objects != null) {
            this.objects.clear();
        }
    }
    
    public String get(final int index) {
        return this.items.get(index);
    }
    
    public void set(final int index, final String newValue) {
        this.items.set(index, newValue);
    }
    
    public Object getObject(final int index) {
        return (this.objects == null) ? null : this.objects.get(index);
    }
    
    public Object getObject(final String item) {
        if (this.objects == null) {
            return null;
        }
        final int index = this.indexOf(item);
        return (index < 0) ? null : this.objects.get(index);
    }
    
    public void setObject(final int index, final Object value) {
        if (value == null && this.objects == null) {
            return;
        }
        if (this.objects == null) {
            this.objects = new ArrayList(this.size());
            for (int i = 0; i < this.size(); ++i) {
                this.objects.add(null);
            }
        }
        this.objects.set(index, value);
    }
    
    public void setObject(final String item, final Object value) {
        int index = this.indexOf(item);
        if (index < 0) {
            this.add(item);
            index = this.size() - 1;
        }
        this.setObject(index, value);
    }
    
    public void assign(final StringList source) {
        if (source.items == null) {
            if (this.items != null) {
                this.items.clear();
            }
        }
        else {
            if (this.items == null) {
                this.items = new ArrayList();
            }
            else {
                this.items.clear();
            }
            this.items.addAll(source.items);
        }
        if (source.objects == null) {
            if (this.objects != null) {
                this.objects.clear();
            }
        }
        else {
            if (this.objects == null) {
                this.objects = new ArrayList();
            }
            else {
                this.objects.clear();
            }
            this.objects.addAll(source.objects);
        }
    }
    
    public void add(final String item) {
        this.items.add(item);
        if (this.objects != null) {
            this.objects.add(null);
        }
    }
    
    public void add(final String item, final Object data) {
        this.items.add(item);
        if (this.objects != null) {
            this.objects.add(null);
        }
        this.setObject(this.items.size() - 1, data);
    }
    
    public void add(final int index, final String item) {
        this.items.add(index, item);
        if (this.objects != null) {
            this.objects.add(index, null);
        }
    }
    
    public void remove(final int index) {
        this.items.remove(index);
        if (this.objects != null) {
            this.objects.remove(index);
        }
    }
    
    public void remove(final String item) {
        int i = this.items.size() - 1;
        while (i >= 0) {
            final String str = this.items.get(i);
            if (str == item || (str != null && str.equals(item))) {
                this.items.remove(i);
                if (this.objects != null) {
                    this.objects.remove(i);
                    break;
                }
                break;
            }
            else {
                --i;
            }
        }
    }
    
    public int indexOf(final String item) {
        return this.items.indexOf(item);
    }
    
    public int find(final String item) {
        int sb = 0;
        int se = this.items.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = compare(item, this.items.get(sp));
            if (flag > 0) {
                sb = sp + 1;
            }
            else {
                if (flag >= 0) {
                    sb = sp;
                    break;
                }
                se = sp - 1;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public int addsort(final String item) {
        final int index = this.find(item);
        if (index < 0) {
            this.items.add(-index - 1, item);
            if (this.objects != null) {
                this.objects.add(-index - 1, null);
            }
        }
        return index;
    }
    
    public int indexOfObject(final Object obj) {
        return (this.objects == null) ? -1 : this.objects.indexOf(obj);
    }
    
    protected void quicksort(int L, final int R) {
        int I;
        do {
            I = L;
            int J = R;
            final String zfn = this.items.get(L + R >> 1);
            while (true) {
                if (compare(zfn, this.items.get(I)) <= 0) {
                    while (compare(zfn, this.items.get(J)) < 0) {
                        --J;
                    }
                    if (I <= J) {
                        Object T = this.items.get(I);
                        this.items.set(I, this.items.get(J));
                        this.items.set(J, T);
                        if (this.objects != null) {
                            T = this.objects.get(I);
                            this.objects.set(I, this.objects.get(J));
                            this.objects.set(J, T);
                        }
                        ++I;
                        --J;
                    }
                    if (I > J) {
                        break;
                    }
                    continue;
                }
                else {
                    ++I;
                }
            }
            if (L < J) {
                this.quicksort(L, J);
            }
        } while ((L = I) < R);
    }
    
    public void sort() {
        if (this.items.size() > 1) {
            this.quicksort(0, this.items.size() - 1);
        }
    }
    
    public String getText() {
        final StringBuffer text = new StringBuffer(1024);
        for (int i = 0; i < this.items.size(); ++i) {
            text.append(this.get(i));
            text.append('\n');
        }
        return text.toString();
    }
    
    public void setText(final String newText) {
        final StringTokenizer st = new StringTokenizer(newText, "\n");
        this.items.clear();
        while (st.hasMoreTokens()) {
            String s = st.nextToken();
            if (s != null && s.length() > 0 && s.charAt(s.length() - 1) == '\r') {
                s = s.substring(0, s.length() - 1);
            }
            this.items.add(s);
        }
        this.objects = null;
    }
    
    public String[] toStringArray() {
        if (this.size() == 0) {
            return null;
        }
        final String[] result = new String[this.size()];
        for (int i = 0; i < this.size(); ++i) {
            result[i] = this.get(i);
        }
        return result;
    }
    
    public Object[] toObjectArray() {
        if (this.size() == 0) {
            return null;
        }
        final Object[] result = new Object[this.size()];
        for (int i = 0; i < this.size(); ++i) {
            result[i] = this.getObject(i);
        }
        return result;
    }
    
    @Override
    public String toString() {
        final StringBuffer result = new StringBuffer("[");
        for (int i = 0; i < this.size(); ++i) {
            result.append("(\"" + this.get(i) + "\", ");
            result.append(this.getObject(i));
            result.append(")");
            if (i < this.size() - 1) {
                result.append(", ");
            }
        }
        result.append("]");
        return result.toString();
    }
    
    public Object clone() {
        StringList result = null;
        try {
            result = (StringList)super.clone();
            result.items = ((this.items == null) ? null : ((List)((ArrayList)this.items).clone()));
            result.objects = ((this.objects == null) ? null : ((List)((ArrayList)this.objects).clone()));
        }
        catch (final CloneNotSupportedException ex) {}
        return result;
    }
    
    public static int compare(final String s1, final String s2) {
        if (s1 == null && s2 == null) {
            return 0;
        }
        if (s1 == null) {
            return -1;
        }
        if (s2 == null) {
            return 1;
        }
        try {
            final String ns1 = new String(s1.toString().getBytes("GB2312"), "ISO-8859-1");
            final String ns2 = new String(s2.toString().getBytes("GB2312"), "ISO-8859-1");
            return ns1.compareTo(ns2);
        }
        catch (final Exception ex) {
            return 0;
        }
    }
    
    public char getDelimiter() {
        return this.getDefined(1) ? this.delimiter : ';';
    }
    
    public void setDelimiter(final char value) {
        this.delimiter = value;
        this.setDefined(1);
    }
    
    public char getQuoteChar() {
        return this.getDefined(2) ? this.quoteChar : '\"';
    }
    
    public void setQuoteChar(final char value) {
        this.quoteChar = value;
        this.setDefined(2);
    }
    
    public char getNameValueSeparator() {
        return this.getDefined(4) ? this.nameValueSeparator : '=';
    }
    
    public void setNameValueSeparator(final char value) {
        this.nameValueSeparator = value;
        this.setDefined(4);
    }
    
    public String getDelimitedText() {
        final StringBuffer retText = new StringBuffer();
        final int count = this.size();
        if (count == 1 && StringUtils.isEmpty(this.get(0))) {
            retText.append(this.getQuoteChar());
            retText.append(this.getQuoteChar());
        }
        else {
            for (int i = 0; i < count; ++i) {
                String s = this.get(i);
                if (s.indexOf(this.getQuoteChar()) != -1 || s.indexOf(this.getDelimiter()) != -1) {
                    s = StringUtils.quote(s, this.getQuoteChar());
                }
                if (i > 0) {
                    retText.append(this.getDelimiter());
                }
                retText.append(s);
            }
        }
        return retText.toString();
    }
    
    public void setDelimitedText(final String value) {
        this.clear();
        if (StringUtils.isEmpty(value)) {
            return;
        }
        StringBuffer buffer = new StringBuffer();
        boolean startQuote = false;
        int i = 0;
        while (i < value.length()) {
            final char curChar = value.charAt(i++);
            if (curChar == this.getQuoteChar()) {
                if (startQuote) {
                    if (i >= value.length()) {
                        break;
                    }
                    if (value.charAt(i) == this.getQuoteChar()) {
                        buffer.append(value.charAt(i));
                        ++i;
                    }
                    else if (value.charAt(i) == this.getDelimiter()) {
                        startQuote = false;
                    }
                    else {
                        buffer.append(value.charAt(i));
                        ++i;
                    }
                }
                else if (buffer.length() > 0) {
                    buffer.append(curChar);
                }
                else {
                    startQuote = true;
                }
            }
            else if (curChar == this.getDelimiter()) {
                if (startQuote) {
                    buffer.append(curChar);
                }
                else {
                    this.add(buffer.toString());
                    buffer = new StringBuffer();
                    startQuote = false;
                }
            }
            else {
                buffer.append(curChar);
            }
        }
        this.add(buffer.toString());
    }
    
    protected String extractName(final String s) {
        if (s == null) {
            return null;
        }
        final int index = s.indexOf(this.getNameValueSeparator());
        if (index == -1) {
            return null;
        }
        return s.substring(0, index);
    }
    
    public String getName(final int index) {
        return this.extractName(this.get(index));
    }
    
    public int indexOfName(final String name) {
        if (name == null) {
            return -1;
        }
        for (int i = 0; i < this.size(); ++i) {
            if (compare(name, this.getName(i)) == 0) {
                return i;
            }
        }
        return -1;
    }
    
    public String getValue(final String name) {
        final int index = this.indexOfName(name);
        if (index == -1) {
            return null;
        }
        return this.get(index).substring(name.length() + 1);
    }
    
    public void setValue(final String name, final String value) {
        final int index = this.indexOfName(name);
        if (StringUtils.isEmpty(value)) {
            if (index != -1) {
                this.remove(index);
            }
        }
        else if (index == -1) {
            this.add(String.valueOf(name) + this.getNameValueSeparator() + value);
        }
        else {
            this.set(index, String.valueOf(name) + this.getNameValueSeparator() + value);
        }
    }
    
    public String getValue(final int index) {
        final String name = this.getName(index);
        if (StringUtils.isEmpty(name)) {
            return this.get(index);
        }
        return this.get(index).substring(name.length() + 1);
    }
    
    public void setValue(final int index, final String value) {
        if (StringUtils.isEmpty(value)) {
            if (index >= 0) {
                this.remove(index);
            }
        }
        else if (index >= 0) {
            this.set(index, String.valueOf(this.getName(index)) + this.getNameValueSeparator() + value);
        }
        else {
            this.add(String.valueOf(this.getNameValueSeparator()) + value);
        }
    }
}
